package com.bonait.bnframework.modules.welcome.activity;

import android.Manifest;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.GridLayoutManager;

import android.view.KeyEvent;

import com.bonait.bnframework.R;
import com.bonait.bnframework.business.MainInit;
import com.bonait.bnframework.common.base.BaseActivity;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.constant.Constants;
import com.bonait.bnframework.common.utils.DisplayManager;
import com.bonait.bnframework.common.utils.ScreenUtils;
import com.bonait.bnframework.manager.ActivityLifecycleManager;
import com.lzy.okgo.OkGo;

import okhttp3.OkHttpClient;
import pub.devrel.easypermissions.AfterPermissionGranted;
import pub.devrel.easypermissions.EasyPermissions;

public class WelcomeActivity extends BaseActivity {

    private Handler handler = new Handler();
    /*
    * 启动模式：
    * 1：启动界面时间与App加载时间相等
    * 2：设置启动界面2秒后跳转
    * */
    private final static int SELECT_MODE = 2;
    private OkHttpClient.Builder builder;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_welcome);
        initWelcome();
        DisplayManager.setReferenceMaxWidthHeight(1208, 800);
        DisplayManager.updateDisplayInfo(this);
    }

    private void initWelcome() {
        switch (WelcomeActivity.SELECT_MODE) {
            case 1:
                fastWelcome();
                break;
            case 2:
                slowWelcome();
                break;
        }
    }


    /*方法1：启动界面时间与App加载时间相等*/
    private void fastWelcome() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                //耗时任务，比如加载网络数据
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        //判断token
                        doPost();
                    }
                });
            }
        }).start();
    }

    /*方法2：设置启动界面2秒后跳转*/
    private void slowWelcome() {
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                //判断token
                doPost();
            }
        }, 2000);
    }

    /**
     * 检查权限是否授权
     */
    public void PermissionCheck() {

        String[] params = {Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.RECORD_AUDIO, Manifest.permission.CAMERA};
        //String[] params = {Manifest.permission.WRITE_EXTERNAL_STORAGE};
        //判断是否获取权限
        if (EasyPermissions.hasPermissions(this, params)) {
            // 全部权限申请成功后
            //1.直接进入登录界面
            skipToLoginActivity();
        } else {
            //未获取权限或拒绝权限时
            EasyPermissions.requestPermissions(this,
                    "小炒设备需要用到以下权限：\n\n1. 文件读取存储权限\n\n2. 录制视频权限\n\n3. 录制音频权限",
                    Constants.ALL_PERMISSION, params);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == Constants.ALL_PERMISSION) {
            //1.直接进入登录界面
            skipToLoginActivity();
        }
    }

    /**
     * 请求服务器判断token是否过期
     */
    private void doPost() {
        PermissionCheck();
//        //判断是否用户修改过密码
//        boolean isChangePwd = PreferenceUtils.getBoolean(SPConstants.CHANGE_PWD, false);
//        if (isChangePwd) {
//            //若修改过密码，则使token失效
//            PreferenceUtils.setString(SPConstants.TOKEN, "");
//        }
//        String token = PreferenceUtils.getString(SPConstants.TOKEN, "");
//        // 请求后台判断token
//        String url = Constants.SERVICE_IP + "/checkToken.do";
//
//        // 修改请求超时时间为6s，与全局超时时间分开
//        builder = new OkHttpClient.Builder();
//        builder.readTimeout(2000, TimeUnit.MILLISECONDS);
//        builder.writeTimeout(2000, TimeUnit.MILLISECONDS);
//        builder.connectTimeout(2000, TimeUnit.MILLISECONDS);
//
//        OkGo.<BaseCodeJson<Void>>post(url)
//                .tag(this)
//                .client(builder.build())
//                .params("token", token)
//                .execute(new JsonCallback<BaseCodeJson<Void>>() {
//                    @Override
//                    public void onSuccess(Response<BaseCodeJson<Void>> response) {
//
//                        // 判断是否开启跳转到测试界面
//                        if (Constants.SKIP_TO_TEST_ACTIVITY) {
//                            skipToTestActivity();
//                        } else {
//                            skipToMainActivity();
//                        }
//
//                    }
//
//                    @Override
//                    public void onError(Response<BaseCodeJson<Void>> response) {
//                        super.onError(response);
//                        skipToLoginActivity();
//                    }
//                });
    }

    /**
     * 跳转到登录页面
     * */
    private void skipToLoginActivity() {
        //初始化
        MainInit.Init(ConfigName.getInstance().app);
        ConfigName.getInstance().IsPortraitScreen=ScreenUtils.IsPortraitScreen(this);

        // 跳转到登录页面
        Intent intent = new Intent(WelcomeActivity.this, LoginActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        // 结束所有Activity
        ActivityLifecycleManager.get().finishAllActivity();
    }

    /**
     * 屏蔽物理返回键
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onDestroy() {
        if(builder!=null)
        {
            OkGo.cancelAll(builder.build());
        }

        if (handler != null) {
            //If token is null, all callbacks and messages will be removed.
            handler.removeCallbacksAndMessages(null);
        }
        super.onDestroy();
    }

}
